@@ -11,6 +11,7 @@
 #include "r_interface/prior_specification.hpp"
 #include "r_interface/list_io.hpp"
 #include "r_interface/boom_r_tools.hpp"
+#include "utils.h"
 
 #include "Models/Glm/BinomialLogitModel.hpp"
 #include "Models/Glm/PosteriorSamplers/BinomialLogitCompositeSpikeSlabSampler.hpp"
@@ -24,43 +25,6 @@
 #include "Rinternals.h"
 #include "R.h"
 
-namespace {
-// Set the coefficients equal to their initial values, and determine
-// which coefficients are initially excluded (i.e. forced to zero).
-// Args:
-//   initial_beta:  Vector containing initial coefficients.
-//   prior_inclusion_probabilities: Prior probabilities that each
-//     coefficient is nonzero.
-//   model:  The model that owns the coefficients.
-//   sampler:  The sampler that will make posterior draws for the model.
-inline void InitializeCoefficients(
-    const BOOM::Vector &initial_beta,
-    const BOOM::Vector &prior_inclusion_probabilities,
-    BOOM::Ptr<BOOM::BinomialLogitModel> model,
-    BOOM::Ptr<BOOM::BinomialLogitCompositeSpikeSlabSampler> sampler) {
-  model->set_Beta(initial_beta);
-  if (min(prior_inclusion_probabilities) >= 1.0) {
-    // Ensure all coefficients are included if you're not going to
-    // do model averaging.
-    sampler->allow_model_selection(false);
-    model->coef().add_all();
-  } else {
-    // Model averaging is desired.  "Small" coefficients start off
-    // excluded from the model.  Large ones start off included.
-    // Adding or dropping is idempotent, so no need to worry about
-    // dropping an already excluded coefficient.
-    for (int i = 0; i < initial_beta.size(); ++i) {
-      if (fabs(initial_beta[i]) < 1e-8) {
-        model->coef().drop(i);
-      } else {
-        model->coef().add(i);
-      }
-    }
-  }
-}
-
-}  // unnamed namespace
-
 extern "C" {
   using BOOM::SpdMatrix;
   using BOOM::Matrix;
@@ -128,10 +92,11 @@ extern "C" {
         sampler->limit_model_selection(prior.max_flips());
       }
 
-      InitializeCoefficients(BOOM::ToBoomVector(r_beta0),
-                             prior.prior_inclusion_probabilities(),
-                             model,
-                             sampler);
+      BOOM::spikeslab::InitializeCoefficients(
+          BOOM::ToBoomVector(r_beta0),
+          prior.prior_inclusion_probabilities(),
+          model,
+          sampler);
 
       int niter = Rf_asInteger(r_niter);
       BOOM::RListIoManager io_manager;
